# Written by jg9000  (jg9000 at gmail.com)
#
# You may freely modify and distribute this program, as long
# as I am credited.

import Tkinter
import threading, Queue
import widgets

class Dialog(Tkinter.Toplevel):
    'Modified version of tkSimpleDialog.Dialog'

    def __init__(self, parent, title = None, resizable=True, icon=None,
                 modal=True):

        '''Initialize a dialog.

        Arguments:

            parent -- a parent window (the application window)

            title -- the dialog title
        '''
        Tkinter.Toplevel.__init__(self, parent)
        self.transient(parent)

        if title:
            self.title(title)

        self.parent = parent

        self.result = None

        self.bodyframe = Tkinter.Frame(self)
        self.initial_focus = self.body(self.bodyframe)
        self.bodyframe.grid(row=0, padx=5, pady=5, sticky='nesw')
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)

        self.buttonbox()

        if self.parent is not None:
            self.geometry("+%d+%d" % (parent.winfo_rootx()+50,
                                      parent.winfo_rooty()+50))
        if not resizable:
            self.resizable(width=False, height=False)

        if icon:
            #if we set the icon too early we get a flashing window
            self.update_idletasks()
            self.iconbitmap(icon)
        self.wait_visibility() # window needs to be visible for the grab

        self.old_grabber = None
        if modal:
            self.old_grabber = self.grab_current()
            self.grab_set()

        if not self.initial_focus:
            self.initial_focus = self

        self.protocol("WM_DELETE_WINDOW", self.cancel)

        self.initial_focus.focus_set()
        if self.initial_focus is self:
            #select first control that isn't a radio button
            self.update_idletasks()
            #self.tk_focusNext().focus_set()
            first_focus = fixed_tk_focusNext(self)
            if first_focus:
                w = first_focus
                while w.winfo_class() == 'Radiobutton':
                    w = fixed_tk_focusNext(w)
                    if w == first_focus:
                        break
                else:
                    w.focus_set()

    def destroy(self):
        '''Destroy the window'''
        self.initial_focus = None
        Tkinter.Toplevel.destroy(self)
        if self.old_grabber:
            self.old_grabber.grab_set()

    def run(self):
        self.wait_window(self)
        return self.result

    #
    # construction hooks

    def body(self, master):
        '''create dialog body.

        return widget that should have initial focus.
        This method should be overridden, and is called
        by the __init__ method.
        '''
        pass

    def buttonbox(self):
        '''add standard button box.

        override if you do not want the standard buttons
        '''

        box = Tkinter.Frame(self)

        w = Tkinter.Button(box, text="OK", width=10, command=self.ok,
                           default=Tkinter.ACTIVE)
        w.pack(side=Tkinter.LEFT, padx=5, pady=5)
        w = Tkinter.Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=Tkinter.LEFT, padx=5, pady=5)

        self.bind("<Return>", self.onReturnKey)
        self.bind("<Escape>", self.cancel)

        box.grid(row=1)
        self.buttonFrame = box

    def onReturnKey(self, event):
        if event.widget and event.widget.winfo_class()=='Button':
            event.widget.invoke()
        elif event.widget and event.widget.winfo_class()=='Text':
            pass #the text widget handles the enter key
        else:
            self.ok()

    def ok(self):
        if not self.validate():
            #self.initial_focus.focus_set()
            return

        self.withdraw()
        self.update_idletasks()
        self.apply()
        self.close()

    def cancel(self, event=None):
        self.close()        

    def close(self):
        if self.parent is not None:
            self.parent.focus_set()
        self.destroy()

    #
    # command hooks

    def validate(self):
        '''validate the data

        This method is called automatically to validate the data before the
        dialog is destroyed. By default, it always validates OK.
        '''

        return 1 # override

    def apply(self):
        '''process the data

        This method is called automatically to process the data, *after*
        the dialog is destroyed. By default, it does nothing.
        '''

        pass # override

class MeterDialog(Dialog):
    def __init__(self, parent, title, workobj, useThread=False,
                 onProgress=None, icon=None, **kwArgs):
        self.workobj = workobj
        self.useThread = useThread
        self.onProgress = onProgress
        Dialog.__init__(self, parent, title, resizable=False, icon=icon, **kwArgs)
        self.result = False
        
    def body(self, master):
        self.meter = widgets.Meter(master, value=0.0)
        self.meter.pack()

    def buttonbox(self):
        box = Tkinter.Frame(self)

        w = Tkinter.Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=Tkinter.LEFT, padx=5, pady=5)
        self.bind("<Escape>", self.cancel)

        box.grid(row=1)

    def run(self):
        self.result = False
        self.needClose = False
        if self.useThread:
            self.queue = Queue.Queue()
            self.startThread()
            self.periodicCall() #start polling the queue
            self.wait_window(self)
        else:
            self.result = self.doWork()
            self.close()
        return self.result

    def periodicCall(self):
        try:
            while True:
                (msg, arg) = self.queue.get_nowait()
                if msg=='done':
                    self.result = True
                    self.close()
                    return
                elif msg=='progress':
                    self.setProgressNow(arg)
        except Queue.Empty:
            pass
        if not self.needClose:
            self.after(100, self.periodicCall)

    def startThread(self):
        self.thread = threading.Thread(target=self.threadProc)
        self.thread.setDaemon(1)
        self.thread.start()

    def threadProc(self):
        try :
            self.doWork()
        except :
            self.needClose = True
        self.queue.put_nowait(('done', 0))

    def setProgressNow(self, val):
        self.meter.set(val)
        if self.onProgress:
            self.onProgress(val)

    def setProgress(self, val):
        if self.useThread:
            self.queue.put_nowait(('progress', val))
        else:
            self.setProgressNow(val)

    def doWork(self):
        for val in self.workobj:
            self.setProgress(val)
            if not self.useThread:
                self.update()
            if self.needClose:
                return False
        return True

    def cancel(self, event=None):
        self.needClose = True
        if self.useThread:
            #the thread will end soon... Don't wait for it
            self.close()


#fix bug in Tkinter's tk_focusNext method:
def fixed_tk_focusNext(anywidget):
    """Return the next widget in the focus order which follows
    widget which has currently the focus.

    The focus order first goes to the next child, then to
    the children of the child recursively and then to the
    next sibling which is higher in the stacking order.  A
    widget is omitted if it has the takefocus resource set
    to 0."""
    name = anywidget.tk.call('tk_focusNext', anywidget._w)
    if not name: return None
    if hasattr(name,'string'):
        name = name.string
    return anywidget._nametowidget(name)
